// Generated by psc version 0.8.0.0

/**
 *  | This module defines helper functions for working with `Bind` instances.
 */
"use strict";
var Prelude = require("../Prelude");

/**
 *  | Forwards Kleisli composition.
 *  |
 *  | For example:
 *  |
 *  | ```purescript
 *  | import Data.Array (head, tail)
 *  |
 *  | third = tail >=> tail >=> head
 *  | ```
 */
var $greater$eq$greater = function (dictBind) {
    return function (f) {
        return function (g) {
            return function (a) {
                return Prelude[">>="](dictBind)(f(a))(g);
            };
        };
    };
};

/**
 *  | A version of `(>>=)` with its arguments flipped.
 */
var $eq$less$less = function (dictBind) {
    return function (f) {
        return function (m) {
            return Prelude[">>="](dictBind)(m)(f);
        };
    };
};

/**
 *  | Backwards Kleisli composition.
 */
var $less$eq$less = function (dictBind) {
    return function (f) {
        return function (g) {
            return function (a) {
                return $eq$less$less(dictBind)(f)(g(a));
            };
        };
    };
};

/**
 *  | Collapse two applications of a monadic type constructor into one.
 */
var join = function (dictBind) {
    return function (m) {
        return Prelude[">>="](dictBind)(m)(Prelude.id(Prelude.categoryFn));
    };
};

/**
 *  | Execute a monadic action if a condition holds.
 *  |
 *  | For example:
 *  |
 *  | ```purescript
 *  | main = ifM ((< 0.5) <$> random)
 *  |          (trace "Heads")
 *  |          (trace "Tails")
 *  | ```
 */
var ifM = function (dictBind) {
    return function (cond) {
        return function (t) {
            return function (f) {
                return Prelude[">>="](dictBind)(cond)(function (cond$prime) {
                    if (cond$prime) {
                        return t;
                    };
                    if (!cond$prime) {
                        return f;
                    };
                    throw new Error("Failed pattern match at Control.Bind line 44, column 1 - line 45, column 1: " + [ cond$prime.constructor.name ]);
                });
            };
        };
    };
};
module.exports = {
    ifM: ifM, 
    join: join, 
    "<=<": $less$eq$less, 
    ">=>": $greater$eq$greater, 
    "=<<": $eq$less$less
};
