// Generated by psc version 0.8.0.0

/**
 *  | Unsafe helper functions for working with immutable arrays.
 *  |
 *  | _Note_: these functions should be used with care, and may result in unspecified
 *  | behavior, including runtime exceptions.
 */
"use strict";
var $foreign = require("./foreign");
var Prelude = require("../Prelude");
var Data_Array = require("../Data.Array");

/**
 *  | Get all but the first element of a non-empty array.
 *  |
 *  | Running time: `O(n)`, where `n` is the length of the array.
 */
var tail = function (xs) {
    return Data_Array.slice(1)(Data_Array.length(xs))(xs);
};

/**
 *  | Get the last element of a non-empty array.
 *  |
 *  | Running time: `O(1)`.
 */
var last = function (xs) {
    return $foreign.unsafeIndex(xs)(Data_Array.length(xs) - 1);
};

/**
 *  | Get all but the last element of a non-empty array.
 *  |
 *  | Running time: `O(n)`, where `n` is the length of the array.
 */
var init = function (xs) {
    return Data_Array.slice(0)(Data_Array.length(xs) - 1)(xs);
};

/**
 *  | Get the first element of a non-empty array.
 *  |
 *  | Running time: `O(1)`.
 */
var head = function (xs) {
    return $foreign.unsafeIndex(xs)(0);
};
module.exports = {
    init: init, 
    last: last, 
    tail: tail, 
    head: head, 
    unsafeIndex: $foreign.unsafeIndex
};
